<?php
/** 
*
* @package phpBB3
* @version $Id: smartfeed_url.php, v 2.0.0 Alpha Version 1 2008/01/08 20:49:00 mhamill@computer.org Exp $
* @copyright (c) Mark D. Hamill (mhamill@computer.org, http://phpbb.potomactavern.org) 
* @license http://opensource.org/licenses/gpl-license.php GNU Public License 
*
*/

/**
* @ignore
*/

// Written by Mark D. Hamill, mhamill@computer.org, http://phpbb.potomactavern.org
// This software is designed to work with phpBB Version 3.0 Gold

// This is the user interface for the Smartfeed software. Users can use it to create a URL they can use 
// with newsfeed readers like FeedReader or web based personal news aggregators like Google Reader.
// Unlike most RSS newsfeeds this one can dip into member-only and user group permission based
// forums which should not be accessible to the world and return posts in these topics. It handles
// RSS and Atom feeds.

define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup();

// Smartfeed cannot be used with Apache authentication unless the .htaccess file is modified to allow smartfeed.php to bypass
// Apache authentication. If you have done so then set the constant SMARTFEED_APACHE_HTACCESS_ENABLED to true.
if (($config['auth_method'] == 'apache') && !(SMARTFEED_APACHE_HTACCESS_ENABLED))
{
	$msg_text = ($user->data['user_type'] == USER_FOUNDER) ? $user->lang['SMARTFEED_APACHE_AUTHENTICATION_WARNING_ADMIN'] : $user->lang['SMARTFEED_APACHE_AUTHENTICATION_WARNING_REG'];
	msg_handler(E_USER_NOTICE, $msg_text, '', '');
}

// Create a full site URL. Cannot quite do this in constants.php since $config['server_name'] is not defined yet.
$server_protocol = ($config['server_protocol']) ? $config['server_protocol'] : (($config['cookie_secure']) ? 'https://' : 'http://');
$server_name = $config['server_name'];
$server_port = ($config['server_port'] == '80') ? '' : ':' . (int) $config['server_port'];
$script_path = $config['script_path'];
$url = $server_protocol . $server_name . $server_port . $script_path;
$url = (strrpos($url, '/') == (strlen($url)-1)) ? $url : $url . '/';
$registered_user_mode = false;

if (($config['auth_method'] <> 'ldap') && ($user->data['user_id'] <> ANONYMOUS) && (($user->data['user_type'] == USER_NORMAL) || ($user->data['user_type'] == USER_FOUNDER)))
{
	// If the user is registered then great, they can authenticate and see private forums
	$user_password = $user->data['user_password'];
	if (is_null($user->data['user_smartfeed_key']))
	{
		// Generate a smartfeed encryption key. This is a one time action. It is used to authenticate the user when they call smartfeed.php.
		$smartfeed_key = gen_rand_string(32);
		// Store the key
		$sql = 'UPDATE ' . USERS_TABLE . " 
					SET user_smartfeed_key = '$smartfeed_key' 
					WHERE user_id = " . $user->data['user_id'];
		if ( !($result = $db->sql_query($sql)) )
		{
			msg_handler(E_USER_ERROR, 'Error updating user_smartfeed_key', __FILE__, __LINE__);
		}
	}
	else
	{
		$smartfeed_key = $user->data['user_smartfeed_key'];
	}
	$smartfeed_user_id = $user->data['user_id'];
	$encrypted_password = encrypt($user_password, $smartfeed_key);
	$encrypted_password_with_ip = encrypt($user_password . '~' . $HTTP_SERVER_VARS['REMOTE_ADDR'], $smartfeed_key);
	$registered_user_mode = true;
	$template->assign_vars(array('S_NOT_ALLOWED' => 'F'));
}
else
{
	// Public (anonymous) users do not need to authenticate so no encrypted passwords are needed
	$smartfeed_user_id = ANONYMOUS;
	$encrypted_password = 'NONE';
	$encrypted_password_with_ip = 'NONE';
	// Make sure to show a message that the user cannot select from read protected forums
	if ($config['auth_method'] <> 'ldap')
	{
		// Note: If LDAP authentication is enabled, then no users can access private forums as a feed, so this message
		// is not needed.
		$template->assign_vars(array('S_NOT_ALLOWED' => 'T'));
	}
}

// If IP Authentication is required, hide HTML that solicits whether the user wants to use IP authentication or not ... they have no choice!
if (SMARTFEED_REQUIRE_IP_AUTHENTICATION && $registered_user_mode)
{
	$template->assign_block_vars('switch_required_ip_authentication', array());	
}
else if ($registered_user_mode)
{
	$template->assign_block_vars('switch_no_required_ip_authentication', array(
		'L_IP_AUTHENTICATION' => $user->lang['SMARTFEED_IP_AUTH']));	
}

$forum_read_ary = array();
$allowed_forums = array();

// Get a list of forums for which this user has read permissions
if ($config['auth_method'] == 'ldap')

{

	// For LDAP authentication, since there is no PHP method to retrieve the user's password from a LDAP directory (or even a hash), we cannot
	// create an authentication string. Thus we are limited to getting newsfeeds for public forums only.
	
	$template->assign_vars(array('S_LDAP' => 'T'));

	// First, get the user_permissions column for Anonymous
	$sql = 'SELECT user_permissions
				FROM ' . USERS_TABLE . '
				WHERE user_id = ' . ANONYMOUS;
	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);
	$user_permissions = $row['user_permissions'];
	$db->sql_freeresult($result);

	// Begin code stolen and tweaked from auth.php
	
	$sql = 'SELECT auth_option, is_global, is_local
				FROM ' . ACL_OPTIONS_TABLE . '
				ORDER BY auth_option_id';
	if ( !($result = $db->sql_query($sql)) )
	{
		handle_error(SMARTFEED_ACL_OPTIONS_TABLE_ERROR);
	}
	
	$local = 0;
	$acl_options = array();
	$acl = array();
	while ($row = $db->sql_fetchrow($result))
	{
		if ($row['is_local'])
		{
			$acl_options['local'][$row['auth_option']] = $local++;
		}
	}
	$db->sql_freeresult($result);

	$user_permissions = explode("\n", $user_permissions);
	
	foreach ($user_permissions as $f => $seq)
	{
		if ($seq)
		{
			$i = 0;
	
			if (!isset($acl[$f]))
			{
				$acl[$f] = '';
			}
	
			while ($subseq = substr($seq, $i, 6))
			{
				// We put the original bitstring into the acl array
				$acl[$f] .= str_pad(base_convert($subseq, 36, 2), 31, 0, STR_PAD_LEFT);
				$i += 6;
			}
		}
	}
	
	// End code stolen from auth.php

	$f_read_local_index = -1;
	foreach ($acl_options['local'] as $key => $value)
	{
		if ($key == 'f_read')
		{
			$f_read_local_index = $value;
			break;
		}
	}

	// Create the allowed forums array. These are the forums for which this user has read privileges. He can select
	// ONLY from these forums. If they request they can choose from a smaller subset of these forums.
	
	foreach ($acl as $key => $value)
	{
		if (substr($value, $f_read_local_index, 1) == '1')
		{
			$forum_read_ary[$key]['f_read'] = 1;
		}
	}
	
}

else

{
	// If the user can be authenticated then we can get a list of allowed forums very easily by calling acl_getf()
	$forum_read_ary = $auth->acl_getf('f_read');
}

foreach ($forum_read_ary as $forum_id => $allowed)
{
		
	if ($allowed['f_read'])
	{
		// Since this user has read access to this forum, add it to the $allowed_forums array
		$allowed_forums[] = (int) $forum_id;
		
		// Also add to $allowed_forums the parents, if any, of this forum. Actually we have to find the parent's parents, etc., going up as far as necesary because 
		// $auth->act_getf does not return the parents for which the user has access, yet parents must be shown are on the interface
		$there_are_parents = true;
		$this_forum_id = (int) $forum_id;
		
		while ($there_are_parents)
		{
			$sql = 'SELECT parent_id 
						FROM ' . FORUMS_TABLE . "
						WHERE forum_id = $this_forum_id";
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			if ((int) $row['parent_id'] == 0)
			{
				$there_are_parents = false;
			}
			else
			{
				// Do not add this parent to the list of allowed forums if it is already in the array
				if (!in_array((int) $row['parent_id'], $allowed_forums))
				{
					$allowed_forums[] = (int) $row['parent_id'];
				} 
				$this_forum_id = (int) $row['parent_id'];	// Keep looping...
			}
		}
	}
}

// Get a list of forums as they appear on the main index for this user. For presentation purposes indent them so they show the natural phpBB3 hierarchy.
// Indenting is cleverly handled by nesting <div> tags inside of other <div> tags, and the template defines the relative offset (20 pixels).
$sql = 'SELECT forum_name, forum_id, parent_id, forum_type
			FROM ' . FORUMS_TABLE . ' 
			WHERE ' . $db->sql_in_set('forum_id', $allowed_forums) . '
			ORDER BY left_id ASC';
$result = $db->sql_query($sql);

$current_level = 0;			// How deeply nested are we at the moment
$parent_stack = array();	// Holds a stack showing the current parent_id of the forum
$parent_stack[] = 0;		// 0, the first value in the stack, represents the <div_0> element, a container holding all the categories and forums in the template

while ($row = $db->sql_fetchrow($result))
{

	$cat_or_forum = ($row['forum_type'] == 0) ? '(' . $user->lang['FORUM_CAT'] . ')' : '';	// Indicate if this is a category
	
	if ((int) $row['parent_id'] <> (int) end($parent_stack) || (end($parent_stack) == 0))
	{
		if (in_array($row['parent_id'],$parent_stack))
		{
			// If parent is in the stack, then pop the stack until the parent is found, otherwise push stack adding the current parent. This creates a </div>
			while ((int) $row['parent_id'] <> (int) end($parent_stack))
			{
				array_pop($parent_stack);
				$current_level--;
				// Need to close a category level here
				$template->assign_block_vars('forums', array( 
					'DIV_OPEN_CLOSE' => 'C',
					'PRINT' => '0'));
			}
		}
		else
		{
			// If the parent is not in the stack, then push the parent_id on the stack. This is also a trigger to indent the block. This creates a <div>
			array_push($parent_stack, (int) $row['parent_id']);
			$current_level++;
			// Need to add a category level here
			$template->assign_block_vars('forums', array( 
				'DIV_OPEN_CLOSE' => 'O',
				'CAT_ID' => 'div_' . $row['parent_id'],
				'PRINT' => '0'));
		}
	}
	
	// This code prints the forum or category, which will exist inside the previously created <div> block
	$template->assign_block_vars('forums', array( 
		'DIV_OPEN_CLOSE' => 'X',
		'FORUM_NAME' => 'elt_' . (int) $row['forum_id'] . '_' . (int) $row['parent_id'],
		'FORUM_LABEL' => $row['forum_name'],
		'PRINT' => '1',
		'IS_FORUM' => ($row['forum_type'] == FORUM_CAT) ? '0' : '1'));
	
}

$db->sql_freeresult($result);

// Now out of the loop, it is important to remember to close any open <div> tags. Typically there is at least one.
while ((int) $row['parent_id'] <> (int) end($parent_stack))
{
	array_pop($parent_stack);
	$current_level--;
	// Need to close the <div> tag
	$template->assign_block_vars('forums', array( 
		'DIV_OPEN_CLOSE' => 'C',
		'PRINT' => '0'));
}

// Now to output most of the lovely text to the screen
$template->assign_vars(array(
	'L_1_HRS' => $user->lang['SMARTFEED_LAST_1_HOURS'],
	'L_1_HRS_VALUE' => $user->lang['SMARTFEED_LAST_1_HOURS_VALUE'],
	'L_12_HRS' => $user->lang['SMARTFEED_LAST_12_HOURS'],
	'L_12_HRS_VALUE' => $user->lang['SMARTFEED_LAST_12_HOURS_VALUE'],
	'L_15_MIN' => $user->lang['SMARTFEED_LAST_15_MINUTES'],
	'L_15_MIN_VALUE' => $user->lang['SMARTFEED_LAST_15_MINUTES_VALUE'],
	'L_3_HRS' => $user->lang['SMARTFEED_LAST_3_HOURS'],
	'L_3_HRS_VALUE' => $user->lang['SMARTFEED_LAST_3_HOURS_VALUE'],
	'L_30_MIN' => $user->lang['SMARTFEED_LAST_30_MINUTES'],
	'L_30_MIN_VALUE' => $user->lang['SMARTFEED_LAST_30_MINUTES_VALUE'],
	'L_6_HRS' => $user->lang['SMARTFEED_LAST_6_HOURS'],
	'L_6_HRS_VALUE' => $user->lang['SMARTFEED_LAST_6_HOURS_VALUE'],
	'L_ALL_POSTS' => $user->lang['SMARTFEED_ALL_POSTS'],
	'L_ALL_POSTS_VALUE' => $user->lang['SMARTFEED_ALL_POSTS_VALUE'],
	'L_ALL_SUBSCRIBED_FORUMS' => $user->lang['SMARTFEED_ALL_FORUMS'],
	'L_ATOM_10' => $user->lang['SMARTFEED_ATOM_10'],
	'L_BASIC' => $user->lang['SMARTFEED_STYLE_BASIC'],
	'L_BOOKMARKS' => $user->lang['SMARTFEED_USE_BOOKMARKS'],
	'L_COMPACT' => $user->lang['SMARTFEED_STYLE_COMPACT'],
	'L_COUNT_LIMIT' => $user->lang['SMARTFEED_COUNT_LIMIT'],
	'L_DAY' => $user->lang['SMARTFEED_LAST_DAY'],
	'L_DAY_VALUE' => $user->lang['SMARTFEED_LAST_DAY_VALUE'],
	'L_FEED_STYLING' => $user->lang['SMARTFEED_FEED_STYLING'],
	'L_FEED_TYPE' => $user->lang['SMARTFEED_FEED_TYPE'],
	'L_FILTER_FOES' => $user->lang['SMARTFEED_FILTER_FOES'],
	'L_FIRST_POST_ONLY' => $user->lang['SMARTFEED_FIRST_POST_ONLY'],
	'L_FORUM_SELECTION' => $user->lang['SMARTFEED_SELECT_FORUMS'],
	'L_FORUMTOPIC' => $user->lang['SMARTFEED_SORT_FORUM_TOPIC'],
	'L_FORUMTOPIC_DESC' => $user->lang['SMARTFEED_SORT_FORUM_TOPIC_DESC'],
	'L_HTML' => $user->lang['SMARTFEED_STYLE_HTML'],
	'L_LAST_FETCH' => $user->lang['SMARTFEED_SINCE_LAST_VISIT_TEXT'],
	'L_LAST_FETCH_VALUE' => $user->lang['SMARTFEED_SINCE_LAST_VISIT'],
	'L_LASTVISIT' => $user->lang['SMARTFEED_LASTVISIT'],
	'L_LDAP_MSG' => $user->lang['SMARTFEED_LDAP_WARNING'],
	'L_LIMIT' => ($registered_user_mode) ? sprintf($user->lang['SMARTFEED_LIMIT_TEXT_USERS'],(SMARTFEED_DEFAULT_FETCH_TIME_LIMIT / (24 * 60 * 60))) : sprintf($user->lang['SMARTFEED_LIMIT_TEXT_PUBLIC'],(SMARTFEED_DEFAULT_FETCH_TIME_LIMIT / (24 * 60 * 60))),
	'L_MAX_SIZE' => $user->lang['SMARTFEED_MAX_WORDS_WANTED'],
	'L_MAX_WORD_SIZE' => $user->lang['SMARTFEED_MAX_SIZE'],
	'L_MIN_WORD_SIZE' => $user->lang['SMARTFEED_MIN_SIZE'],
	'L_MONTH' => $user->lang['SMARTFEED_LAST_MONTH'],
	'L_MONTH_VALUE' => $user->lang['SMARTFEED_LAST_MONTH_VALUE'],
	'L_NO' => $user->lang['NO'],
	'L_NOT_LOGGED_IN_MSG' => $user->lang['SMARTFEED_NOT_LOGGED_IN'],
	'L_POSTDATE' => $user->lang['SMARTFEED_SORT_POST_DATE'],
	'L_POSTDATE_DESC' => $user->lang['SMARTFEED_SORT_POST_DATE_DESC'],
	'L_PRIVATE_MGS_IN_FEED' => $user->lang['SMARTFEED_PRIVATE_MESSAGES_IN_FEED'],
	'L_PUBLIC_DEFAULT' => ($registered_user_mode) ? '' : ' selected="selected"',
	'L_QUARTER' => $user->lang['SMARTFEED_LAST_QUARTER'],
	'L_QUARTER_VALUE' => $user->lang['SMARTFEED_LAST_QUARTER_VALUE'],
	'L_REMOVE_YOUR_POSTS' => $user->lang['SMARTFEED_REMOVE_YOURS'],
	'L_RESET' => $user->lang['RESET'],
	'L_RSS_10' 	=> $user->lang['SMARTFEED_RSS_10'],
	'L_RSS_20' 	=> $user->lang['SMARTFEED_RSS_20'],
	'L_SMARTFEED_EXPLANATION' => $user->lang['SMARTFEED_EXPLANATION'],
	'L_SMARTFEED_VERSION' => $user->lang['SMARTFEED_VERSION'],
	'L_SORTBY' => $user->lang['SMARTFEED_SORT_BY'],
	'L_STYLING' => $user->lang['SMARTFEED_FEED_STYLING'],
	'L_SUBMIT' => $user->lang['SMARTFEED_GENERATE_URL_TEXT'],
	'L_TOPICSONLY' => $user->lang['SMARTFEED_TOPICS_ONLY'],
	'L_TWO_WEEKS' => $user->lang['SMARTFEED_LAST_TWO_WEEKS'],
	'L_TWO_WEEKS_VALUE' => $user->lang['SMARTFEED_LAST_TWO_WEEKS_VALUE'],
	'L_URL' => $user->lang['SMARTFEED_URL_LABEL'],
	'L_USER' => $user->lang['SMARTFEED_SORT_USER_ORDER'],
	'L_WEEK' => $user->lang['SMARTFEED_LAST_WEEK'],
	'L_WEEK_VALUE' => $user->lang['SMARTFEED_LAST_WEEK_VALUE'],
	'L_YES' => $user->lang['YES'],
	'LA_NO_FORUMS_SELECTED' => $user->lang['SMARTFEED_NO_FORUMS_SELECTED'],
	'LA_SIZE_ERROR' => $user->lang['SMARTFEED_SIZE_ERROR'],
	'LA_SIZE_ERROR_MIN' => $user->lang['SMARTFEED_SIZE_ERROR_MIN'],
	'S_ATOM_10_VALUE' => SMARTFEED_ATOM_10_VALUE,
	'S_PHPEXT' => $phpEx,
	'S_PWD' => $encrypted_password, 
	'S_PWD_WITH_IP' => $encrypted_password_with_ip, 
	'S_RSS_10_VALUE' => SMARTFEED_RSS_10_VALUE,
	'S_RSS_20_VALUE' => SMARTFEED_RSS_20_VALUE,
	'S_SMARTFEED_PAGE_URL' => SMARTFEED_PAGE_URL,
	'S_SMARTFEED_TITLE'	=> SMARTFEED,
	'S_SMARTFEED_VERSION' => SMARTFEED_VERSION,
	'S_USER_ID' => $smartfeed_user_id,
	'UA_SITE_URL' => $url

	)
);

page_header(SMARTFEED);

$template->set_filenames(array(
	'body' => 'smartfeed_url_body.html')
);

make_jumpbox(append_sid("{$phpbb_root_path}viewforum.$phpEx"));

page_footer();

function encrypt($data_input, $key)
{   

	// This function encrypts $data_input with the given $key. It uses the PHP Secure Communications Library, found
	// at http://phpseclib.sourceforge.net/. If mcrypt is compiled into PHP it will use that, otherwise it will use the functions 
	// from this library. This allows for maximum portability of Smartfeed. Thanks to Terrafrost at phpbb.com for the suggestion
	// to use this library!
	
	global $phpbb_root_path;
	static $des;
	
	if (!isset($des))
	{
		include($phpbb_root_path . 'includes/Crypt/TripleDES.php');
    	$des = new Crypt_TripleDES();
	}

    $des->setKey($key);

	// Encrypt
    $encrypted_data = base64_encode($des->encrypt($data_input));
	return $encrypted_data;

}   

?>